import argparse
import csv
import itertools as itt

import flask


def _url_for(route_name, **params):
    return flask.url_for(route_name, **params)


def make_app(csv_path):
    app = flask.Flask(__name__)

    @app.route('/')
    def _index():
        csv_doc = _read_csv_doc(csv_path, 0, -1)
        return flask.render_template('index.html',
                                     csv_path=csv_path,
                                     n_cols=len(csv_doc['col_names']),
                                     table_headers=csv_doc['col_names'],
                                     app_css_url=_url_for('static', filename='app.css'),
                                     app_js_url=_url_for('static', filename='app.js'),
                                     favicon_url=_url_for('static', filename='favicon.png'))

    @app.route('/partial/rows')
    def _partial_rows():
        n_rows = int(flask.request.args.get('n_rows', 10))
        after_id = int(flask.request.args.get('after_id', -1))
        csv_doc = _read_csv_doc(csv_path, n_rows, after_id)
        return flask.render_template('table_rows.html',
                                     rows_values=[(row_id, [row[col]
                                                            for col in csv_doc['col_names']])
                                                  for row_id, row in csv_doc['rows']])

    return app


def _read_csv_doc(path, n_rows, after_id):
    with open(path) as f:
        reader = csv.DictReader(f)
        col_names = reader.fieldnames
        rows = [(e_i + after_id + 1, e)
                for e_i, e in enumerate(itt.islice(reader, after_id+1, after_id+1+n_rows))]

        return {'rows': rows,
                'col_names': col_names}


def make_cli_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('csv_path')
    parser.add_argument('-p', '--port',
                        help='HTTP port to serve on. Defaults to 7482',
                        type=int,
                        default=7482)
    return parser


def main():
    args = make_cli_parser().parse_args()
    app = make_app(args.csv_path)
    app.run(port=args.port, debug=True)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['racks_on_tabs']

package_data = \
{'': ['*'],
 'racks_on_tabs': ['static/app.css',
                   'static/app.css',
                   'static/app.css',
                   'static/app.js',
                   'static/app.js',
                   'static/app.js',
                   'static/favicon.png',
                   'static/favicon.png',
                   'static/favicon.png',
                   'templates/*']}

install_requires = \
['flask>=1.1.2,<2.0.0', 'waitress>=1.4.4,<2.0.0']

entry_points = \
{'console_scripts': ['racks-on-tabs = racks_on_tabs.__main__:main']}

setup_kwargs = {
    'name': 'racks-on-tabs',
    'version': '0.1.2',
    'description': 'Command line app for browsing CSV files with row lazy loading.',
    'long_description': '# racks-on-tabs\n\n> (What you got?) Racks on tabs on tabs\n>\n> (He got) Racks on tabs on tabs\n>\n> (We got) Racks on tabs on taaabs\n>\n> ~[YC & Future, circa 2011, probably](https://www.youtube.com/watch?v=r5w21_Vphbg&ab_channel=ycvevo)\n\nCommand line app for browsing CSV files with row lazy loading.\n\n![](docs/imgs/1m-csv-example.png)\n*This is what opening [1M rows](http://eforexcel.com/wp/downloads-18-sample-csv-files-data-sets-for-testing-sales/) under 100ms looks like.*\n\n```sh\n$ racks-on-tabs <PATH_TO_CSV>\n```\n\nStarts a local webserver that serves the app content, accessible at [localhost:7482](http://localhost:7482).\n\n# Installation\n\nRequires Python 3. Tested with Python 3.8.\n\n## pipx\n\n_racks-on-tabs_ is a Python app with a set of dependencies.\nThe recommended way to install it is via [pipx](https://github.com/pipxproject/pipx):\n\n```sh\n$ pipx install racks-on-tabs\n$ racks-on-tabs <PATH_TO_CSV>\n```\n\n## virtualenv\n\nAlternatively, you can set up a local virtual env and install _racks-on-tabs_ there:\n\n```sh\n$ python3 -m venv venv\n$ source venv/bin/activate\n$ pip3 install racks-on-tabs\n$ racks-on-tabs <PATH_TO_CSV>\n```\n\n## Troubleshooting\n\nIf for some reason you get "executable not found" errors, try locating the library installation path, and run:\n\n```sh\n$ python -m path/to/pip/packages/racks_on_tabs.__main__ <PATH_TO_CSV>\n```\n\n',
    'author': 'Alexander Juda',
    'author_email': 'alexanderjuda@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/alexjuda/racks-on-tabs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
